package pro.litvinovg.w2phtml.gui;

import java.io.File;
import java.net.URI;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.sun.star.document.XDocumentProperties;
import com.sun.star.document.XDocumentPropertiesSupplier;
import com.sun.star.frame.XDesktop;
import com.sun.star.frame.XDispatchProvider;
import com.sun.star.frame.XFrame;
import com.sun.star.frame.XModel;
import com.sun.star.frame.XStorable;
import com.sun.star.lang.XComponent;
import com.sun.star.lang.XMultiComponentFactory;
import com.sun.star.lang.XMultiServiceFactory;
import com.sun.star.lang.XServiceInfo;
import com.sun.star.text.XText;
import com.sun.star.text.XTextDocument;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XComponentContext;
import com.sun.star.util.XModifiable;

public class ODTDocument {

	private static final Logger logger = LoggerFactory.getLogger(ODTDocument.class);

	
	private XComponentContext context;
	private XDesktop xDesktop;
	private XMultiComponentFactory multiComponentFactory;
	private XMultiServiceFactory multiServiceFactory;
	private XComponent currentDocument;
	private XDocumentProperties documentProperties;
	private XDocumentPropertiesSupplier documentPropertiesSupplier;
	private XText text = null;
	private XFrame frame;
	private String fileName = "";
	private XDispatchProvider dispatchProvider;
	XTextDocument textDocument;

	public ODTDocument(String fileName) {
		this.fileName = fileName;
	}
	
	public ODTDocument(XComponentContext componentContext) {
		if (componentContext != null) {
			context = componentContext;
			multiComponentFactory = context.getServiceManager();
			try {
				Object oDesktop = multiComponentFactory.createInstanceWithContext("com.sun.star.frame.Desktop", context);
				xDesktop = UnoRuntime.queryInterface(XDesktop.class, oDesktop);
				currentDocument = xDesktop.getCurrentComponent();
				//Get document readOnly status. If so, abort.
				//Get document path. If doc not saved - abort.
				if (currentDocument == null) {
					return;
				}
				//Get document type. Work only with ODT.
				XModel model  = UnoRuntime.queryInterface(XModel.class, currentDocument);
				if (!isSupported(currentDocument,"com.sun.star.text.TextDocument")) {
					return;
				}
				XTextDocument textDoc = UnoRuntime.queryInterface(XTextDocument.class, currentDocument);
				if (textDoc == null) {
					logger.error("Error. TextDoc is null.");
					return;
				}
				XStorable storable  = UnoRuntime.queryInterface(XStorable.class, textDoc);
				
				/*
				 * if (storable == null || storable.isReadonly()) {
				 * Debug.printLog("Document is read only", componentContext); } else {
				 * Debug.printLog("Document is in rw mode", componentContext); }
				 */
				
				XModifiable modifieable  = UnoRuntime.queryInterface(XModifiable.class, textDoc);

				if (modifieable == null || modifieable.isModified()) {
					logger.error("Document is modified. Save changes before conversion.");
				} else {
					//Debug.printLog("Document isn't modified", componentContext);
				}
				if (!storable.hasLocation()) {
					logger.error("Document is not saved. Save document before conversion.");
					return;
				}
					String url = model.getURL();
					URI uri = new URI(url);
					File file = new File(uri);
					if (file.exists() && file.canRead()) {
						fileName = file.getAbsolutePath();
					} 

			} catch (Throwable e) {
				logger.error(e.toString());
				return;
			}	
		}
		
		
	}
	public String getFileName() {
		return fileName;
	}
	
	private boolean isSupported(Object object, String service) {
		XServiceInfo info = UnoRuntime.queryInterface(XServiceInfo.class, object);
		if (info == null) {
			return false;
		}
		return info.supportsService(service);
	}
}
