package w2phtml.pageSplitters;

import static w2phtml.office.XMLString.TABLE_TABLE_COLUMN;
import static w2phtml.office.XMLString.TABLE_TABLE_COLUMN_GROUP;
import static w2phtml.office.XMLString.TABLE_TABLE_HEADER_COLUMNS;
import static w2phtml.office.XMLString.TABLE_TABLE_HEADER_ROWS;
import static w2phtml.office.XMLString.TABLE_TABLE_ROW;
import static w2phtml.office.XMLString.TABLE_TABLE_ROWS;
import static w2phtml.office.XMLString.TABLE_TABLE_ROW_GROUP;
import static w2phtml.office.XMLString.TEXT_SOFT_PAGE_BREAK;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import w2phtml.office.OfficeReader;

public class TableSplitter extends BasicSplitter implements ISplitter {

	public TableSplitter(OfficeReader officeReader, SplitFactory factory) {
		super(officeReader, factory);
	}

	@Override
	public SplitResults Split(Node table) {

		/*
		 * // TODO: 0.Test if soft-page-break not at start of table // - in that
		 * case just remove it and insert before table // 1.Create new table //
		 * 2.Copy to it table:table-column's and // table:table-header-rows //
		 * 3.Move nodes before soft-page-break to new table //4. IF in one
		 * table:row exist more one Algorithm IF SPB at start - just move it
		 * higher IF SPB between rows - just copy table move row and put SPB
		 * between tables IF SPB inside row, inside cell - copy table, copy
		 * empty row, copy each empty cell and in each cell move every node up
		 * to the first SPB
		 * 
		 * 
		 */
		Node parent = table.getParentNode();
		Node tableFirstPart = table.cloneNode(false);
		NodeList tableChildNodes = table.getChildNodes();
		// Node counter
		int i = 0;
		SplitResults results = new SplitResults(false);
		// Loop through the TABLE:TABLE child nodes
		while (tableChildNodes.getLength() > i) {
			Node tableChildNode = tableChildNodes.item(i);

			if (tableChildNode.getNodeType() == Node.ELEMENT_NODE) {
				//Node name
				String tableChildNodeName = tableChildNode.getNodeName();
				//System.out.println("Table child node " + tableChildNodeName);
				if (containsSPB(tableChildNode)){
					if (tableChildNodeName.equals(TEXT_SOFT_PAGE_BREAK)) {
						// remove inner soft page break node
						table.removeChild(tableChildNode);
						
					} else if (tableChildNodeName.equals(TABLE_TABLE_ROW_GROUP)) {
						if (factory.split(tableChildNode).isDataMoved()){
							results.setDataMoved(true);
							tableFirstPart.appendChild(tableChildNode.getPreviousSibling());
						}
					} else if ( tableChildNodeName.equals(TABLE_TABLE_ROWS)) {
						if (factory.split(tableChildNode).isDataMoved()){
							results.setDataMoved(true);
							tableFirstPart.appendChild(tableChildNode.getPreviousSibling());
						}
					} else if ( tableChildNodeName.equals(TABLE_TABLE_ROW)) {
						if (factory.split(tableChildNode).isDataMoved()){
							results.setDataMoved(true);
							tableFirstPart.appendChild(tableChildNode.getPreviousSibling());
						}
					} else  if (tableChildNodeName.equals(TABLE_TABLE_COLUMN)
							 || tableChildNodeName.equals(TABLE_TABLE_COLUMN_GROUP)
							 || tableChildNodeName.equals(TABLE_TABLE_HEADER_ROWS)
							 || tableChildNodeName.equals(TABLE_TABLE_HEADER_COLUMNS)) {
						//Remove Soft Page Break
						removeSPB(tableChildNode);					
					}
					break;
					
				} else {
					//Before SPB
					//Description nodes
					if (tableChildNodeName.equals(TABLE_TABLE_COLUMN)
					 || tableChildNodeName.equals(TABLE_TABLE_COLUMN_GROUP)
					 || tableChildNodeName.equals(TABLE_TABLE_HEADER_ROWS)
					 || tableChildNodeName.equals(TABLE_TABLE_HEADER_COLUMNS)) {
						//Append to clone table
						tableFirstPart.appendChild(tableChildNode.cloneNode(true));
						//increment counter
						i++;
					} else {
						//Append to clone table
						tableFirstPart.appendChild(tableChildNode);
						results.setDataMoved(true);
					}
				}
			}
		}
		if (results.isDataMoved()) {
			parent.insertBefore(tableFirstPart, table);
		}
		return results;
	}

}
